@@ -252,83 +252,22 @@ static int amdgpu_cgs_get_firmware_info(struct cgs_device *cgs_device,
 
 		if (!adev->pm.fw) {
 			switch (adev->asic_type) {
-			case CHIP_TAHITI:
-				strcpy(fw_name, "/*(DEBLOBBED)*/");
-				break;
-			case CHIP_PITCAIRN:
-				if ((adev->pdev->revision == 0x81) &&
-				    ((adev->pdev->device == 0x6810) ||
-				    (adev->pdev->device == 0x6811))) {
-					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				}
-				break;
-			case CHIP_VERDE:
-				if (((adev->pdev->device == 0x6820) &&
-					((adev->pdev->revision == 0x81) ||
-					(adev->pdev->revision == 0x83))) ||
-				    ((adev->pdev->device == 0x6821) &&
-					((adev->pdev->revision == 0x83) ||
-					(adev->pdev->revision == 0x87))) ||
-				    ((adev->pdev->revision == 0x87) &&
-					((adev->pdev->device == 0x6823) ||
-					(adev->pdev->device == 0x682b)))) {
-					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				}
-				break;
-			case CHIP_OLAND:
-				if (((adev->pdev->revision == 0x81) &&
-					((adev->pdev->device == 0x6600) ||
-					(adev->pdev->device == 0x6604) ||
-					(adev->pdev->device == 0x6605) ||
-					(adev->pdev->device == 0x6610))) ||
-				    ((adev->pdev->revision == 0x83) &&
-					(adev->pdev->device == 0x6610))) {
-					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				}
-				break;
-			case CHIP_HAINAN:
-				if (((adev->pdev->revision == 0x81) &&
-					(adev->pdev->device == 0x6660)) ||
-				    ((adev->pdev->revision == 0x83) &&
-					((adev->pdev->device == 0x6660) ||
-					(adev->pdev->device == 0x6663) ||
-					(adev->pdev->device == 0x6665) ||
-					 (adev->pdev->device == 0x6667)))) {
-					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				} else if ((adev->pdev->revision == 0xc3) &&
-					 (adev->pdev->device == 0x6665)) {
-					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
-				}
-				break;
 			case CHIP_BONAIRE:
 				if ((adev->pdev->revision == 0x80) ||
 					(adev->pdev->revision == 0x81) ||
 					(adev->pdev->device == 0x665f)) {
 					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				}
 				break;
 			case CHIP_HAWAII:
 				if (adev->pdev->revision == 0x80) {
 					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				}
 				break;
 			case CHIP_TOPAZ:
@@ -338,76 +277,76 @@ static int amdgpu_cgs_get_firmware_info(struct cgs_device *cgs_device,
 				    ((adev->pdev->device == 0x6900) && (adev->pdev->revision == 0xD1)) ||
 				    ((adev->pdev->device == 0x6900) && (adev->pdev->revision == 0xD3))) {
 					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				} else
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_TONGA:
 				if (((adev->pdev->device == 0x6939) && (adev->pdev->revision == 0xf1)) ||
 				    ((adev->pdev->device == 0x6938) && (adev->pdev->revision == 0xf1))) {
 					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				} else
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_FIJI:
-				strcpy(fw_name, "/*(DEBLOBBED)*/");
+				strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_POLARIS11:
 				if (type == CGS_UCODE_ID_SMU) {
 					if (ASICID_IS_P21(adev->pdev->device, adev->pdev->revision)) {
 						info->is_kicker = true;
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					} else if (ASICID_IS_P31(adev->pdev->device, adev->pdev->revision)) {
 						info->is_kicker = true;
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					} else {
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					}
 				} else if (type == CGS_UCODE_ID_SMU_SK) {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				}
 				break;
 			case CHIP_POLARIS10:
 				if (type == CGS_UCODE_ID_SMU) {
 					if (ASICID_IS_P20(adev->pdev->device, adev->pdev->revision)) {
 						info->is_kicker = true;
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					} else if (ASICID_IS_P30(adev->pdev->device, adev->pdev->revision)) {
 						info->is_kicker = true;
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					} else {
-						strcpy(fw_name, "/*(DEBLOBBED)*/");
+						strscpy(fw_name, "/*(DEBLOBBED)*/");
 					}
 				} else if (type == CGS_UCODE_ID_SMU_SK) {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				}
 				break;
 			case CHIP_POLARIS12:
 				if (ASICID_IS_P23(adev->pdev->device, adev->pdev->revision)) {
 					info->is_kicker = true;
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				} else {
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				}
 				break;
 			case CHIP_VEGAM:
-				strcpy(fw_name, "/*(DEBLOBBED)*/");
+				strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_VEGA10:
 				if ((adev->pdev->device == 0x687f) &&
 					((adev->pdev->revision == 0xc0) ||
 					(adev->pdev->revision == 0xc1) ||
 					(adev->pdev->revision == 0xc3)))
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				else
-					strcpy(fw_name, "/*(DEBLOBBED)*/");
+					strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_VEGA12:
-				strcpy(fw_name, "/*(DEBLOBBED)*/");
+				strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			case CHIP_VEGA20:
-				strcpy(fw_name, "/*(DEBLOBBED)*/");
+				strscpy(fw_name, "/*(DEBLOBBED)*/");
 				break;
 			default:
 				DRM_ERROR("SMC firmware not supported\n");
